#!/usr/bin/perl

######################################################################
package Cognition::uF::hAlarm;
######################################################################

use CGI::Util;
use Cognition::HTMLParser::Utils;
use Cognition::uF;
use XML::LibXML qw(:all);
use strict;

sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @nodes = searchClass('valarm', $within);
	my @rv;
	
	foreach my $ev (@nodes)
	{
		push @rv, parse($page, $ev);
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	
	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	push @rv, Cognition::HTMLParser::Utils::BNodeURI($this->{_dom}, 'Alarm')
		unless (@rv);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub rdf_subject
{
	my $this    = shift;
	my $ns      = 'urn:ietf:rfc:2445#';
	my $CLASS   = shift;

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
		Cognition::RDFModel::Value->new($ns.'Valarm', 1));
	
	# Patterns for simple data.
	my @_SINGLES  = qw(summary trigger description duration action repeat);
	my @_SINGLESR = qw(parent);
	my @_MULTIS   = qw();
	my @_MULTISR  = qw(child sibling);
	my @_OBJECTS  = qw(attendee attach);
	
	# Properties which exist as singles.
	foreach my $property (@_SINGLES)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property});
		if ($property =~ /^repeat/)
			{ $val->setDataType('http://www.w3.org/2000/01/rdf-schema#Integer'); }
		$subject->addVal($ns.$property, $val);
	}

	# Resource-type properties which exist as singles.
	foreach my $property (@_SINGLESR)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property}, 1);
		$subject->addVal($ns.$property, $val);
	}
		
	# Resource-type properties which exist as multiple values.
	foreach my $property (@_MULTISR)
	{
		foreach my $V (@{ $this->{$property} })
		{
			next unless (length $V);
			my $val = Cognition::RDFModel::Value->new($V, 1);
			$subject->addVal($ns.$property, $val);
		}
	}

	# Embedded objects which may be represented as resources
	foreach my $property (@_OBJECTS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			my $val;
			if (ref $V && $V->can('uri'))
				{ $val = Cognition::RDFModel::Value->new($V->uri, 1); }
			else
				{ $val = Cognition::RDFModel::Value->new($V); }
			$subject->addVal($ns.$property, $val);
		}
	}
	
	return $subject;
	
} #/sub rdf_subject


sub metadata_ns
{
	my $this = shift;
	return { 'ical' => { nsurl=>'urn:ietf:rfc:2445#',title=>'iCalendar' } };
} #/sub metadata_ns


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	my $event_node = $rv->{'_dom'}->cloneNode(1);
	
	my $id = $event_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	data_patterns($page, $event_node, 4);
	
	simple_parse(
		$rv,
		$event_node,
		[
			['action',       '?'],
			['attach',       'U?'],
			['attendee',     'M*', {'embedded'=>'hCard', 'is-in-cal'=>1}],
			['description',  '?'],
			['duration',     'D?'],
			['repeat',       '?'],
			['trigger',      'D?'] # TODO: should really allow 'related' 
			                       # subproperty and allow datetime values too.
		],
		{
			'rel-enclosure' => 'attach'
		}
	);

	bless $rv;
	return $rv;
	
} #/sub parse


1; #/package Cognition::uF::hAlarm




