#!/usr/bin/perl

######################################################################
package Cognition::uF::hResume;
######################################################################

use CGI::Util;
use Cognition::HTMLParser::Utils;
use Cognition::uF;
use XML::LibXML qw(:all);
use strict;


sub consume
{
	my $page        = shift;
	$page->{uF}->{hResume} = [ parse_all($page) ];
	foreach my $r (@{ $page->{uF}->{hResume} })
		{ $r->rdf_subject_merge($page); }
	if (defined $page->{uF}->{hResume}->[0])
		{ $page->mergeNS($page->{uF}->{hResume}->[0]); }
}


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @rv;
	
	my @nodes = searchClass('hresume', $within);
	foreach my $a (@nodes)
	{
		my $A = parse($page, $a);
		push @rv, $A;
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	return $this->{contact}->uri($all);
} #/sub uri


sub rdf_subject_merge
{
	my $this    = shift;
	my $page    = shift;
	my $val;

	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	$subject->addVal('http://purl.org/dc/terms/abstract',
		Cognition::RDFModel::Value->new($this->{summary}))
		if (length $this->{summary});
		
	foreach my $prop (qw(education experience))
	{
		foreach my $x (@{$this->{$prop}})
		{
			my $X = Cognition::RDFModel::Subject->new($x->uri);
			$X->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
				Cognition::RDFModel::Value->new('http://ramonantonio.net/doac/0.1/'.ucfirst($prop), 1));
			$X->addVal('http://ramonantonio.net/doac/0.1/date-starts',
				Cognition::RDFModel::Value->new($x->{dtstart}))
				if ($x->{dtstart});
			$X->addVal('http://ramonantonio.net/doac/0.1/date-ends',
				Cognition::RDFModel::Value->new($x->{dtend}))
				if ($x->{dtend});
			$X->addVal('http://ramonantonio.net/doac/0.1/'.($prop eq 'education' ? 'subject' : 'activity'),
				Cognition::RDFModel::Value->new($x->{description}))
				if ($x->{description});
			foreach my $l (@{$x->{location}})
			{
				if (UNIVERSAL::can($l, 'uri'))
				{
					$X->addVal('http://ramonantonio.net/doac/0.1/location',
						Cognition::RDFModel::Value->new($l->uri, 1));
				}
				else
				{
					$X->addVal('http://ramonantonio.net/doac/0.1/location',
						Cognition::RDFModel::Value->new($l));
				}
			}
			if ($x->{associated_hcard})
			{
				foreach my $o (@{$x->{associated_hcard}->{org}})
				{
					$X->addVal('http://ramonantonio.net/doac/0.1/organization',
						Cognition::RDFModel::Value->new($o->uri, 1))
				}
				foreach my $t (@{$x->{associated_hcard}->{title}})
				{
					$X->addVal('http://ramonantonio.net/doac/0.1/'.($prop eq 'education' ? 'title' : 'position'),
						Cognition::RDFModel::Value->new($t));
				}
				$subject->addVal('http://purl.org/uF/hCard/terms/hasHistoricCard',
					Cognition::RDFModel::Value->new($x->{associated_hcard}->uri, 1));
			}
			
			$page->mergeSubject($X);
			$subject->addVal('http://ramonantonio.net/doac/0.1/'.$prop,
				Cognition::RDFModel::Value->new($x->uri, 1));
		}
	}
	
	foreach my $x (@{$this->{skill}})
	{
		$subject->addVal('http://ramonantonio.net/doac/0.1/skill',
			Cognition::RDFModel::Value->new($x));
	}
	
	foreach my $x (@{$this->{contact}->{lang}})
	{
		my $Xuri = Cognition::HTMLParser::Utils::BNodeURI(undef, 'LanguageSkill');
		my $X = Cognition::RDFModel::Subject->new($Xuri);
		$X->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
			Cognition::RDFModel::Value->new('http://ramonantonio.net/doac/0.1/LanguageSkill', 1));
		$X->addVal('http://ramonantonio.net/doac/0.1/language',
			Cognition::RDFModel::Value->new($x));

		$page->mergeSubject($X);
		$subject->addVal('http://ramonantonio.net/doac/0.1/skill',
			Cognition::RDFModel::Value->new($Xuri, 1));
	}

	foreach my $x (@{$this->{publication}})
	{
		$subject->addVal('http://ramonantonio.net/doac/0.1/publication',
			Cognition::RDFModel::Value->new($x));
	}
	
	foreach my $x (@{$this->{affiliation}})
	{
		my $X = Cognition::RDFModel::Subject->new($x->uri);
		$X->addVal('urn:ietf:rfc:2426#member',
			Cognition::RDFModel::Value->new($this->{contact}->uri, 1));

		$page->mergeSubject($X);
	}

	$page->mergeSubject($subject);
	
} #/sub rdf_subject


sub metadata_ns
{
	my $this = shift;
	return {
		'doac' => {
				nsurl=>'http://ramonantonio.net/doac/0.1/',
				title=>'Description of a Career'
			}
		};
} #/sub metadata_ns


sub parse
{
	my $page  = shift;
	my $rv    = { '_dom'=>shift };
	my $root  = $rv->{'_dom'}->cloneNode(1);
	my $root2 = $rv->{'_dom'}->cloneNode(1);
	
	my $id = $root->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	data_patterns($page, $root,  2);
	data_patterns($page, $root2, 2);
	
	simple_parse(
		$rv,
		$root,
		[
			['summary',     '?'],
			['contact',     'm1', {'embedded'=>'hCard'}],
			['education',   'm*', {'embedded'=>'hEvent'}],
			['experience',  'm*', {'embedded'=>'hEvent'}],
			['skill',       '*'],
			['affiliation', 'M*', {'embedded'=>'hCard'}],
			['cite',        't',  {'use-key'=>'publication'}]
		]
	);
	
	simple_parse(
		$rv,
		$root2,
		[
			['experience',  'm*', {'embedded'=>'hCard','use-key'=>'experience-hcard'}],
			['education',   'm*', {'embedded'=>'hCard','use-key'=>'education-hcard'}]
		]
	);
	
	foreach my $prop (qw(education experience))
	{
		foreach my $e ( @{$rv->{$prop}} )
		{
			foreach my $ehc ( @{$rv->{$prop.'-hcard'}} )
			{
				my $ehcxp = $ehc->{parent_property_node}->getAttribute('_xpath');
				if ($ehcxp eq $e->{'_dom'}->getAttribute('_xpath'))
					{ $e->{'associated_hcard'} = $ehc; }
			}
		}
	}
	
	bless $rv;
	return $rv;
	
} #/sub parse


1; #/package Cognition::uF::hResume

